/*
 * Balancing_Robot.c
 *
 * Created: 06/05/2014 18:13:20
 *  Author: Zenios Agapiou
 */ 


#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>



#define MOTOR1A PORTD6
#define MOTOR1B PORTD7
#define MOTOR2A PORTB0
#define MOTOR2B PORTD5

static int uart_putchar(char c, FILE *stream);
int uart_getchar(FILE *stream);
char USARTReadChar();
void USARTInit(uint16_t ubrr_value);

FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

int main(void)
{
	/*ENABLE EXTERNAL INTERRUPT FOR PROGRAMMING*/
	DDRD &= ~(1<<PORTD3) | (1<<PORTD4);
	PORTD |= (1<<PORTD3) | (1<<PORTD4);
	EICRA |= (1<<ISC11) | (0<< ISC10);
	EIMSK|= (1<<INT1);

	DDRB |= (1<<PORTB1) | (1<<MOTOR2A) | (1<<PORTB5); // ENABLE OC2A AND MOTOR CONTROL
	DDRD |= (1<<PORTB2) | (1<<MOTOR2B) | (1<<MOTOR1A) | (1<<MOTOR1B); // ENABLE OC2B AND MOTOR CONTROLS
	PORTB |= (1<<MOTOR2A); // DIRECTION
	PORTD |= (1<<MOTOR1A); //DIRECTION
	PORTD &= ~((1<< MOTOR1B)|(1<<MOTOR2B)); // DIRECTION
	
	/* TIMER 2*/
	/* PWM FREQUENCY = 16MHz / PRESCALER * 510 = 31372 KHz*/
	/* Enable Correct Phase PWM, Compare match goes high, top goes low*/

	TCCR1A |= (1<<WGM10) | (0<<WGM11) | (1<<COM1A1) |(1<<COM1A0) | (1<<COM1B1) | (1<<COM1B0);
	//TCCR2B |= (1<<WGM22);
	TCNT1 = 0;
	
	OCR1A = 100; //
	OCR1B = 100;
	
	TCCR1B |= (1<<CS10); //START TIMER2 - PWM
	
	stdout = &mystdout;
	
	USARTInit(25);
	
	
	
	sei();
	
	printf("Hello\n");
	
    while(1)
    {
		char test = USARTReadChar();
		
		if (test == '-'){
				printf("decreasing motors");
			OCR1A-= 1;
			OCR1B -=1;
		}
		else if (test == '+'){
			printf("increasing motors");
			OCR1A +=1;
			OCR1B +=1;
		}
		else if (test == 's'){
			OCR1A = 0; //
			OCR1B = 0;
			printf("Stopping motors");
		}
		else if (test == 'a'){
			OCR1A = 100; //
			OCR1B = 100;
			printf("Starting motors");
		}
		else
		{
			printf("Stop typing bullshit");
		}
		
		
    }
}



static int uart_putchar (char c, FILE *stream)
{
	if (c == '\n')
	uart_putchar('\r', stream);
	
	// Wait for the transmit buffer to be empty
	while(!(UCSR0A & (1<<UDRE0)));

	UDR0=c;
	
	return 0;
}

int uart_getchar(FILE * stream)
{
	//Wait until the transmitter is ready

	while(!(UCSR0A & (1<<UDRE0)))
	{}
	
	char data = UDR0;
	
	uart_putchar(data, stream); //Send to console what has been received, so we can see when typing
	
	return data;

}

void USARTInit(uint16_t ubrr_value)
{

   //Set Baud rate
   UBRR0L = ubrr_value;
   UBRR0H = (ubrr_value>>8);

   /*Set Frame Format
   >> Asynchronous mode
   >> No Parity
   >> 1 StopBit
   >> char size 8
   */
   UCSR0C = (3<<UCSZ00);
   
   //Enable The receiver and transmitter
   UCSR0B=(1<<RXEN0)|(1<<TXEN0);

}

char USARTReadChar()
{
	//Wait untill a data is available

	while(!(UCSR0A & (1<<RXC0)))
	{
		//Do nothing
	}

	//Now USART has got data from host
	//and is available is buffer

	return UDR0;
	
}



ISR (INT1_vect)
{
	/* IF PORTD4 IS LOW*/
	if ( (PORTD & PORTD4) != (1<<PORTD4)){
		
	}
	
	PORTB |= (1<< PORTB5);
	PORTB &= ~(1<< PORTB4);
	cli();
	
	
	wdt_reset();
	wdt_enable(WDTO_15MS);
		
	
	while(1);
}